﻿// -----------------------------------------------------------------------
// <copyright file="CSVResults.cs" company=".">
// 
// </copyright>
// -----------------------------------------------------------------------

namespace mockupCirene.Model.Components
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using mockupCirene.Helpers;

    /// <summary>
    /// All the data from csv file is obtained and stored in separate varibles and obtained through properties.
    /// </summary>
    public class CSVResults : ObservableObject , IComparable
    {
        int _ID;

        /// <summary>
        /// Property that gets and sets test ID.
        /// </summary>
        public int ID
        {
            get { return _ID; }
            set 
            { 
                _ID = value;
                RaisePropertyChanged("ID");
            }
        }


        string _testName;

        /// <summary>
        /// Property that gets and sets test name.
        /// </summary>
        public string TestName
        {
            get { return _testName; }
            set 
            { 
                _testName = value;
                RaisePropertyChanged("TestName");
            }
        }

        string _appName;

        /// <summary>
        /// Property that gets and sets application name.
        /// </summary>
        public string AppName
        {
            get { return _appName; }
            set { _appName = value;
            RaisePropertyChanged("AppName");
            }
        }

        int _cumulativeTraceLength;

        /// <summary>
        /// Gets and sets value of cumulative trace length.
        /// </summary>
        public int CumulativeTraceLength
        {
            get { return _cumulativeTraceLength; }
            set 
            {
                _cumulativeTraceLength = value;
                RaisePropertyChanged("CumulativeTraceLength");
            }
        }

        int _traceLength;

        /// <summary>
        /// Gets and sets value of trace length.
        /// </summary>
        public int TraceLength
        {
            get { return _traceLength; }
            set 
            { 
                _traceLength = value;
                RaisePropertyChanged("TraceLength");
            }
        }

        double _coverage;

        /// <summary>
        /// Gets and sets coverage percentage.
        /// </summary>
        public double Coverage
        {
            get { return _coverage; }
            set 
            { 
                _coverage = value;
                RaisePropertyChanged("Coverage");
            }
        }

        int _runtime;

        /// <summary>
        /// Gets and sets runtime.
        /// </summary>
        public int Runtime
        {
            get { return _runtime; }
            set 
            { 
                _runtime = value;
                RaisePropertyChanged("Runtime");
            }
        }

        List<OldInstrumentedMethod> _coveredMethods;

        /// <summary>
        /// Gets and sets values from list of covered, instrumented methods.
        /// </summary>
        public List<OldInstrumentedMethod> CoveredMethods
        {
            get 
            {
                if (_coveredMethods == null) 
                {
                    CoveredMethods = new List<OldInstrumentedMethod>();
                }
                return _coveredMethods; 
            }
            set 
            { 
                _coveredMethods = value;
                RaisePropertyChanged("CoveredMethods");
            }
        }

        List<OldInstrumentedMethod> _notCoveredMethods;

        /// <summary>
        /// Gets and sets  values from list of not covered, instrumented methods.
        /// </summary>
        public List<OldInstrumentedMethod> NotCoveredMethods
        {
            get 
            {
                if (_notCoveredMethods == null)
                {
                    NotCoveredMethods = new List<OldInstrumentedMethod>();
                }
                return _notCoveredMethods; 
            }
            set 
            { 
                _notCoveredMethods = value;
                RaisePropertyChanged("NotCoveredMethods");
            }
        }
        ObservableCollection<int> _covered;

        /// <summary>
        /// Gest and sets the number of covered and not covered methods.
        /// </summary>
        public ObservableCollection<int> ListCoverage
        {
            get { return new ObservableCollection<int>() { CoveredMethods.Count, NotCoveredMethods.Count }; }
            set 
            { 
                _covered = value;
                RaisePropertyChanged("Covered");
            }
        }
        decimal _coveragePercentage;
        
        /// <summary>
        /// Gets and sets the coverage percentage, calculating the value.
        /// </summary>
        public decimal CoveragePercentage
        {
            get { return (CoveredMethods.Count * 100 / (CoveredMethods.Count + NotCoveredMethods.Count)); }
            set 
            { 
                _coveragePercentage = value;
                RaisePropertyChanged("CoveragePercentage");
            }
        }

        /// <summary>
        /// Method for object comparison.
        /// </summary>
        /// <param name="obj"></param>
        /// <returns>
        /// Returning the compared cumulative trace length.
        /// </returns>
        public int CompareTo(object obj)
        {        
            return this.CumulativeTraceLength.CompareTo((obj as CSVResults).CumulativeTraceLength);
        }
    }
}

