﻿// -----------------------------------------------------------------------
// <copyright file="IPAddress.cs" company=".">
// 
// </copyright>
// -----------------------------------------------------------------------

namespace mockupCirene.Model
{
    using System;
    using System.ComponentModel;
    using mockupCirene.Helpers;

    /// <summary>
    /// With this class, all the information related to IP address is included.
    /// </summary>
    [Serializable]
    public class IPAddress : ObservableObject, IDataErrorInfo
    {
        byte _firstOctet;

        /// <summary>
        /// Gets and sets the first octet of the IP address.
        /// </summary>
        public byte FirstOctet
        {
            get { return _firstOctet; }
            set
            {
                _firstOctet = value;
                RaisePropertyChanged("FirstOctet");
            }
        }

        byte _secondOctet;

        /// <summary>
        /// Gets and sets the second octet of the IP address.
        /// </summary>
        public byte SecondOctet
        {
            get { return _secondOctet; }
            set
            {
                _secondOctet = value;
                RaisePropertyChanged("SecondOctet");
            }
        }

        byte _thirdOctet;

        /// <summary>
        /// Gets and sets the third octet of the IP address.
        /// </summary>
        public byte ThirdOctet
        {
            get { return _thirdOctet; }
            set
            {
                _thirdOctet = value;
                RaisePropertyChanged("ThirdOctet");
            }
        }

        byte _fourthOctet;

        /// <summary>
        /// Gets and sets the fourth octet of the IP address.
        /// </summary>
        public byte FourthOctet
        {
            get { return _fourthOctet; }
            set
            {
                _fourthOctet = value;
                RaisePropertyChanged("FourthOctet");
            }
        }

        /// <summary>
        /// Overrides the ToString method.
        /// </summary>
        /// <returns>
        /// Returns the IP address.
        /// </returns>
        public override string ToString()
        {
            return Address;
        }

        /// <summary>
        /// Gets the IP address in a specific format.
        /// </summary>
        public string Address
        {
            get { return string.Format("{0}.{1}.{2}.{3}", FirstOctet, SecondOctet, ThirdOctet, FourthOctet); }
        }

        /// <summary>
        /// Gets an error of no implementation by throwing an exception.
        /// </summary>
        public string Error
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Depending on the column name, this method obtains the correct octet.
        /// </summary>
        /// <param name="columnName"></param>
        /// <returns>
        /// Returns the given octet, or a warning message.
        /// </returns>
        public string this[string columnName]
        {
            get
            {
                string result = string.Empty;

                switch (columnName)
                {
                    case "FirstOctet":
                        if (FirstOctet > 255)
                        {
                            FirstOctet = 255;
                            result = "Value cannot be bigger then 255";
                        }
                        break;
                    case "SecondOctet":
                        if (SecondOctet > 255)
                        {
                            SecondOctet = 255;
                            result = "Value cannot be bigger then 255";
                        }
                        break;
                    case "ThirdOctet":
                        if (ThirdOctet > 255)
                        {
                            ThirdOctet = 255;
                            result = "Value cannot be bigger then 255";
                        }
                        break;
                    case "FourthOctet":
                        if (FourthOctet > 255)
                        {
                            FourthOctet = 255;
                            result = "Value cannot be bigger then 255";
                        }
                        break;
                };

                return result;

            }
        }
    }
}
