﻿// -----------------------------------------------------------------------
// <copyright file="InstrumentedMethods.cs" company=".">
// 
// </copyright>
// -----------------------------------------------------------------------

namespace mockupCirene.Model
{
    using System.Collections.Generic;
    using System.Collections.ObjectModel;

    /// <summary>
    /// This class implements some methods for handling and manipulation of methods IDs.
    /// </summary>
    public class InstrumentedMethods : IList<InstrumentedMethod>
    {
        ObservableCollection<InstrumentedMethod> _instrumentedMethods;

        private ObservableCollection<InstrumentedMethod> InternalInstrumentedMethods
        {
            get
            {
                if (_instrumentedMethods == null) 
                {
                    InternalInstrumentedMethods = new ObservableCollection<InstrumentedMethod>();
                }
                return _instrumentedMethods; 
            }
            set { _instrumentedMethods = value; }
        }

        /// <summary>
        /// Finds the index of the current instrumented method.
        /// </summary>
        /// <param name="item"></param>
        /// <returns>
        /// Returns the index of the instrumented method.
        /// </returns>
        public int IndexOf(InstrumentedMethod item)
        {
            return InternalInstrumentedMethods.IndexOf(item);
        }

        /// <summary>
        /// Inserts the specified method at the given index.
        /// </summary>
        /// <param name="index"></param>
        /// <param name="item"></param>
        public void Insert(int index, InstrumentedMethod item)
        {
            InternalInstrumentedMethods.Insert(index, item);
        }

        /// <summary>
        /// Removes the specified method at the given index.
        /// </summary>
        /// <param name="index"></param>
        public void RemoveAt(int index)
        {
            InternalInstrumentedMethods.RemoveAt(index);
        }

        /// <summary>
        /// Gets and sets instrumented methods at the sepecific index.
        /// </summary>
        /// <param name="index"></param>
        /// <returns>
        /// Returns instrumented methods at the sepecific index.
        /// </returns>
        public InstrumentedMethod this[int index]
        {
            get
            {
                return InternalInstrumentedMethods[index];
            }
            set
            {
                InternalInstrumentedMethods[index] = value;
            }
        }

        /// <summary>
        /// Adds instrumented method into the collection.
        /// </summary>
        /// <param name="item"></param>
        public void Add(InstrumentedMethod item)
        {
            InternalInstrumentedMethods.Add(item);
        }

        /// <summary>
        /// Clear the collection.
        /// </summary>
        public void Clear()
        {
            InternalInstrumentedMethods.Clear();
        }

        /// <summary>
        /// Determines whether the instrumented method is in the collection.
        /// </summary>
        /// <param name="item"></param>
        /// <returns>
        /// Returns true if method was found in the collection.
        /// </returns>
        public bool Contains(InstrumentedMethod item)
        {
            return InternalInstrumentedMethods.Contains(item);
        }

        /// <summary>
        /// Copies the collection to an array, from the specified position.
        /// </summary>
        /// <param name="array"></param>
        /// <param name="arrayIndex"></param>
        public void CopyTo(InstrumentedMethod[] array, int arrayIndex)
        {
            InternalInstrumentedMethods.CopyTo(array, arrayIndex);
        }

        /// <summary>
        /// Gets the number of methods in the collection.
        /// </summary>
        public int Count
        {
            get { return InternalInstrumentedMethods.Count; }
        }

        /// <summary>
        /// Returns false if it is read-only.
        /// </summary>
        public bool IsReadOnly
        {
            get { return false; }
        }

        /// <summary>
        /// Removes the first method found in the collection.
        /// </summary>
        /// <param name="item"></param>
        /// <returns>
        /// Returns the removed method.
        /// </returns>
        public bool Remove(InstrumentedMethod item)
        {
            return InternalInstrumentedMethods.Remove(item);
        }

        /// <summary>
        /// Obtains the enumerator that iterates through the collection of methods.
        /// </summary>
        /// <returns>
        /// Returns the current iterator of the collection.
        /// </returns>
        public IEnumerator<InstrumentedMethod> GetEnumerator()
        {
            return InternalInstrumentedMethods.GetEnumerator();
        }

        /// <summary>
        /// Obtains the enumerator that iterates through the collection of interface.
        /// </summary>
        /// <returns>
        /// Returns the current iterator of the collection.
        /// </returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return InternalInstrumentedMethods.GetEnumerator();
        }

    }
}
