﻿// -----------------------------------------------------------------------
// <copyright file="TestApplication.cs" company=".">
// 
// </copyright>
// -----------------------------------------------------------------------

namespace mockupCirene.Model
{
    using System.Collections.Generic;

    /// <summary>
    /// Obtains the covered and not covered methods names and calculates coverage information.
    /// </summary>
    public class TestApplication : TestBase
    {
        List<InstrumentedMethod> _covered;

        /// <summary>
        /// Gets and sets the list of covered methods.
        /// </summary>
        public List<InstrumentedMethod> Covered
        {
            get
            {
                if (_covered == null)
                {
                    Covered = new List<InstrumentedMethod>();
                }
                return _covered;
            }
            set
            {
                _covered = value;
                RaisePropertyChanged("Covered");
            }
        }

        List<InstrumentedMethod> _notCovered;

        /// <summary>
        /// Gets and sets the list of not covered methods.
        /// </summary>
        public List<InstrumentedMethod> NotCovered
        {
            get
            {
                if (_notCovered == null)
                {
                    NotCovered = new List<InstrumentedMethod>();
                }
                return _notCovered;
            }
            set
            {
                _notCovered = value;
                RaisePropertyChanged("NotCovered");
            }
        }

        /// <summary>
        /// Overrides the get and set method for covered method number.
        /// </summary>
        public override int CoveredNumber
        {
            get
            {
                return Covered.Count;
            }
            set
            {
                base.CoveredNumber = value;
            }
        }

        /// <summary>
        /// Overrides the get method for not covered method number.
        /// </summary>
        public override int NotCoveredNumber
        {
            get
            {
                return NotCovered.Count;
            }
        }

        /// <summary>
        /// Overrides the get method for coverage calculation.
        /// </summary>
        public override double Coverage
        {
            get
            {
                return (double)Covered.Count / ((double)(Covered.Count + NotCovered.Count));
            }
        }
    }
}
