﻿// -----------------------------------------------------------------------
// <copyright file="TestBase.cs" company=".">
// 
// </copyright>
// -----------------------------------------------------------------------

namespace mockupCirene.Model
{
    using System;
    using System.Xml.Serialization;
    using mockupCirene.Helpers;
    using mockupCirene.Model.Components;
    

    /// <summary>
    /// This class contains get and set methods for implemented properties for testing results information.
    /// </summary>
    [Serializable]
    public abstract class TestBase : ObservableObject
    {
        int _id;

        /// <summary>
        /// Gets and sets the ID of a test case.
        /// </summary>
        public virtual int ID
        {
            get { return _id; }
            set 
            { 
                _id = value;
                RaisePropertyChanged("ID");
            }
        }

        int _traceLength;

        /// <summary>
        /// Gets and sets the trace length gained by the executed test case.
        /// </summary>
        public virtual int TraceLength
        {
            get { return _traceLength; }
            set
            {
                _traceLength = value;
                RaisePropertyChanged("TraceLength");
            }
        }

        double _coverage;

        /// <summary>
        /// Gets and sets coverage value for the current application. 
        /// </summary>
        public virtual double Coverage
        {
            get { return _coverage; }
            set
            {
                _coverage = value;
                RaisePropertyChanged("Coverage");
            }
        }

        int _runtime;

        /// <summary>
        /// Gets and sets the runtime of the test case.
        /// </summary>
        public virtual int Runtime
        {
            get { return _runtime; }
            set
            {
                _runtime = value;
                RaisePropertyChanged("Runtime");
            }
        }

        string _name;

        /// <summary>
        /// Gets and sets the name of the reached method.
        /// </summary>
        public virtual string Name
        {
            get { return _name; }
            set 
            { 
                _name = value;
                RaisePropertyChanged("Name");
            }
        }

        int _coveredNumber = -1;

        /// <summary>
        /// Gets and sets the number of covered methods.
        /// </summary>
        public virtual int CoveredNumber
        {
            get { return _coveredNumber; }
            set 
            { 
                _coveredNumber = value;
                RaisePropertyChanged("CoveredNumber");
            }
        }

        int  _notCoveredNumber = -1;

        /// <summary>
        /// Gets and sets the number of not covered methods.
        /// </summary>
        public virtual int NotCoveredNumber
        {
            get { return _notCoveredNumber; }
            set 
            { 
                _notCoveredNumber = value;
                RaisePropertyChanged("NotCoveredNumber");
            }
        }

        Array _coverageSeries;

        /// <summary>
        /// Gets and sets the covered and not covered number of methods for one coverage series.
        /// </summary>
        [XmlIgnore]
        public Array CoverageSeries
        {
            get
            {
                return new CovererageItem[] { new CovererageItem() { Name = "Covered", Count = CoveredNumber }, new CovererageItem() { Name = "NotCovered", Count = NotCoveredNumber } };
            }
            set
            {
                _coverageSeries = value;
                RaisePropertyChanged("CoverageSeries");
            }
        }


    }
}
