﻿// -----------------------------------------------------------------------
// <copyright file="TSTTest.cs" company=".">
// 
// </copyright>
// -----------------------------------------------------------------------

namespace mockupCirene.Model
{
    using System.Collections.Generic;
    using mockupCirene.Model.Components;

    /// <summary>
    /// Holds all the necessary information of covered and not covered methods, but also calculates coverage percentage.
    /// </summary>
    public class TestCase : TestBase
    {
        List<InstrumentedMethod> _covered;

        /// <summary>
        /// Gets and sets the covered method item of a list.
        /// </summary>
        public List<InstrumentedMethod> Covered
        {
            get
            {
                if (_covered == null)
                {
                    Covered = new List<InstrumentedMethod>();
                }
                return _covered;
            }
            set
            {
                _covered = value;
                RaisePropertyChanged("Covered");
            }
        }

        List<InstrumentedMethod> _notCovered;

        /// <summary>
        /// Gets and sets the not covered method item of a list.
        /// </summary>
        public List<InstrumentedMethod> NotCovered
        {
            get
            {
                if (_notCovered == null)
                {
                    NotCovered = new List<InstrumentedMethod>();
                }
                return _notCovered;
            }
            set
            {
                _notCovered = value;
                RaisePropertyChanged("NotCovered");
            }
        }

        private List<InstrumentedMethod> _cumulativeCovered;

        /// <summary>
        /// Gets and sets the cumulative covered method item of a list.
        /// </summary>
        public List<InstrumentedMethod> CumulativeCovered
        {
            get 
            {
                if (_cumulativeCovered == null) 
                {
                    CumulativeCovered = new List<InstrumentedMethod>();
                }
                return _cumulativeCovered; }
            set
            {
                _cumulativeCovered = value;
                RaisePropertyChanged("CumulativeCovered");
            }
        }

        private List<InstrumentedMethod> _cumulativeNotCovered;

        /// <summary>
        /// Gets and sets the cumulative not covered method item of a list.
        /// </summary>
        public List<InstrumentedMethod> CumulativeNotCovered
        {
            get
            {
                if (_cumulativeNotCovered == null) 
                {
                    CumulativeNotCovered = new List<InstrumentedMethod>();
                }
                return _cumulativeNotCovered; }
            set
            {
                _cumulativeNotCovered = value;
                RaisePropertyChanged("CumulativeNotCovered");
            }
        }

        TestResult _result;

        /// <summary>
        /// Gets and sets the result of the execution.
        /// </summary>
        public TestResult Result
        {
            get { return _result; }
            set
            {
                _result = value;
                RaisePropertyChanged("Result");
            }
        }

        TestApplications _applications;

        /// <summary>
        /// Gets and sets applications object of class TestApplications.
        /// </summary>
        public TestApplications Applications
        {
            get
            {
                if (_applications == null)
                {
                    Applications = new TestApplications();
                }
                return _applications;
            }
            set
            {
                _applications = value;
                RaisePropertyChanged("Applications");
            }
        }

        /// <summary>
        /// Overrides get and set method of the trace length. 
        /// </summary>
        public override int TraceLength
        {
            get
            {
                int trace = 0;
                foreach (var item in Applications)
                {
                    trace += item.TraceLength;

                }
                return trace;
            }
            set
            {
                base.TraceLength = value;
                RaisePropertyChanged("TraceLength");
            }
        }

        /// <summary>
        /// Calculates and gets the full coverage percentage. 
        /// </summary>
        public double FullCoverage
        {
            get { return 100 * (double)(CumulativeCovered.Count / (double)(CumulativeCovered.Count + CumulativeNotCovered.Count)); }
        }

        private double _coverage;

        /// <summary>
        /// Overrides the get and set method for coverage.
        /// </summary>
        public override double Coverage
        {
            get
            {
                return _coverage;
            }
            set
            {
                _coverage = value;
                RaisePropertyChanged("Coverage");
            }
        }
 
        /// <summary>
        /// Overrides the get and set method for covered method number.
        /// </summary>
        public override int CoveredNumber
        {
            get
            {
                if (base.CoveredNumber == -1)
                {                  
                    ExtractCoveredNotCovered();
                }
                return base.CoveredNumber;
            }
            set
            {
                base.CoveredNumber = value;
                RaisePropertyChanged("CoveredNumber");
            }
        }

        /// <summary>
        /// Overrides the get and set method for not covered method number.
        /// </summary>
        public override int NotCoveredNumber
        {
            get
            {
                if (base.CoveredNumber == -1)
                {                  
                    ExtractCoveredNotCovered();
                }
                return base.NotCoveredNumber;
            }
            set
            {
                base.NotCoveredNumber = value;
                RaisePropertyChanged("NotCoveredNumber");
            }
        }

        /// <summary>
        /// Method that extracts covered and not covered method number for each application.
        /// </summary>
        private void ExtractCoveredNotCovered()
        {
            base.CoveredNumber = 0;
            base.NotCoveredNumber = 0;
            foreach (var item in Applications)
            {
                base.CoveredNumber += item.CoveredNumber;
                base.NotCoveredNumber += item.NotCoveredNumber;
            }
        }
    }
}
