﻿
namespace mockupCirene.Model
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Data.OleDb;
    using System.IO;
    using System.Linq;
    using System.Runtime.InteropServices;
    using System.Text;
    using System.Xml;
    using System.Xml.Serialization;
    using mockupCirene.Model.Components;

    /// <summary>
    /// Main model for whole application.
    /// </summary>    
    [Serializable]
    public class TestCases : TestBase
    {
        #region Properties

        #region first view

        private string _projectName;

        /// <summary>
        /// Gets and sets the project name currently being used.
        /// </summary>
        [XmlElement]
        public string ProjectName
        {
            get { return _projectName; }
            set
            {
                _projectName = value;
                RaisePropertyChanged("ProjectName");
            }
        }

        LogLevel _testLogLevel;

        /// <summary>
        /// Gets and sets the testing log level.
        /// </summary>
        [XmlElement]
        public LogLevel TestLogLevel
        {
            get { return _testLogLevel; }
            set
            {
                _testLogLevel = value;
                RaisePropertyChanged("TestLogLevel");
            }
        }

        /// <summary>
        /// Retrieves an array of the log level states.
        /// </summary>
        public IEnumerable<LogLevel> LogLevels
        {
            get { return Enum.GetValues(typeof(LogLevel)).Cast<LogLevel>(); }
        }

        string _testDirectory;

        /// <summary>
        /// Gets and sets the test directory.
        /// </summary>
        [XmlElement]
        public string TestDirectory
        {
            get
            {
                return _testDirectory;
            }
            set
            {
                _testDirectory = value;
                RaisePropertyChanged("TestDirectory");
            }
        }

        string _resultLocation;

        /// <summary>
        /// Gets and sets the project directory.
        /// </summary>
        [XmlElement]
        public string ProjectDirectory
        {
            get
            {
                return _resultLocation;
            }
            set
            {
                _resultLocation = value;
                RaisePropertyChanged("ProjectDirectory");
            }
        }

        string _outputDirectory;

        /// <summary>
        /// Gets and sets the output directory.
        /// </summary>
        [XmlElement]
        public string OutputDirectory
        {
            get { return _outputDirectory; }
            set
            {
                _outputDirectory = value;
                RaisePropertyChanged("OutputDirectory");
            }
        }

        string _resultCsvFile;

        /// <summary>
        /// Gets and sets the path of the resul.csv file.
        /// </summary>
        [XmlElement]
        public string ResultCsvFile
        {
            get
            {
                if (_resultCsvFile == null)
                {
                    _resultCsvFile = OutputDirectory + @"\result.csv";
                }
                return _resultCsvFile;
            }
            set
            {
                _resultCsvFile = value;
            }
        }

     
        IPAddress _hostAddress;

        /// <summary>
        /// Gets and sets the IP address.
        /// </summary>
        public IPAddress HostAddress
        {
            get
            {
                if (_hostAddress == null)
                {
                    HostAddress = new IPAddress();
                }
                return _hostAddress;
            }
            set
            {
                _hostAddress = value;
                RaisePropertyChanged("HostAddress");
            }
        }

        string _coverageConfiguration;

        /// <summary>
        /// Gets and sets the coverage configuration.
        /// </summary>
        public string CoverageConfiguration
        {
            get { return _coverageConfiguration; }
            set
            {
                _coverageConfiguration = value;
                RaisePropertyChanged("CoverageConfiguration");
            }
        }

        string _SelectionAndPrioritizationConfig;

        /// <summary>
        /// Gets and sets the configuration file for selection and prioritization.
        /// </summary>
        public string SelectionAndPrioritizationConfigFile
        {
            get { return _SelectionAndPrioritizationConfig; }
            set
            {
                _SelectionAndPrioritizationConfig = value;
                RaisePropertyChanged("SelectionAndPrioritizationConfigFile");
            }
        }

        string _allTestCase;

        /// <summary>
        /// Gets and sets the criteria for selection of all test cases.
        /// </summary>
        public string AllTestCases
        {
            get { return _allTestCase; }
            set
            {
                _allTestCase = value;
                RaisePropertyChanged("AllTestCases");
            }
        }

        string _methodsDirectory;

        /// <summary>
        /// Gets and sets the directory to the methods file.
        /// </summary>
        public string MethodsDirectory
        {
            get
            {
                return _methodsDirectory;               
            }
            set
            {
                _methodsDirectory = value;
                RaisePropertyChanged("MethodsDirectory");
            }
        }

        string _testCases;

        /// <summary>
        /// Gets and sets the file that holds the changed functions list.
        /// </summary>
        public string ChangedFunctionsFile
        {
            get { return _testCases; }
            set
            {
                _testCases = value;
                RaisePropertyChanged("ChangedFunctionsFile");
            }
        }

        string _outputTestPlan;

        /// <summary>
        /// Gets and sets the output test plan file.
        /// </summary>
        public string OutputTestPlan
        {
            get { return _outputTestPlan; }
            set
            {
                _outputTestPlan = value;
                RaisePropertyChanged("OutputTestPlan");
            }
        }

        PrioritizationCriteria _prioritization;

        /// <summary>
        /// Gets and sets the criteria for test cases prioritization.
        /// </summary>
        public PrioritizationCriteria Prioritization
        {
            get { return _prioritization; }
            set
            {
                _prioritization = value;
                RaisePropertyChanged("Prioritization");
            }
        }

        /// <summary>
        /// Gets a certain value from the prioritization criteria enumeration.
        /// </summary>
        public IEnumerable<PrioritizationCriteria> PrioritizationCriterias
        {
            get { return Enum.GetValues(typeof(PrioritizationCriteria)).Cast<PrioritizationCriteria>(); }
        }

        SelectionCriteria _criteria;

        /// <summary>
        /// Gets and sets the test case selection criteria.
        /// </summary>
        public SelectionCriteria Selection
        {
            get { return _criteria; }
            set
            {
                _criteria = value;
                RaisePropertyChanged("Selection");
            }
        }

        /// <summary>
        /// Gets a certain value from the selection criteria enumeration.
        /// </summary>
        public IEnumerable<SelectionCriteria> SelectionCriterias
        {
            get { return Enum.GetValues(typeof(SelectionCriteria)).Cast<SelectionCriteria>(); }
        }

        #endregion

        #region second view

        private string _firstScript;

        /// <summary>
        /// Gets and sets the first executable script.
        /// </summary>
        public string FirstScript
        {
            get { return _firstScript; }
            set
            {
                _firstScript = value;
                RaisePropertyChanged("FirstScript");
            }
        }

        private string _apkFileName;

        /// <summary>
        /// Gets and sets apk file name.
        /// </summary>
        public string ApkFileName
        {
            get { return _apkFileName; }
            set
            {
                _apkFileName = value;
                RaisePropertyChanged("ApkFileName");
            }
        }

        private string _secondScript;

        /// <summary>
        /// Gets and sets the second executable script.
        /// </summary>
        public string SecondScript
        {
            get { return _secondScript; }
            set
            {
                _secondScript = value;
                RaisePropertyChanged("SecondScript");
            }
        }

        private string _packageName;

        /// <summary>
        /// Gets and sets the application package name.
        /// </summary>
        public string PackageName
        {
            get { return _packageName; }
            set
            {
                _packageName = value;
                RaisePropertyChanged("PackageName");
            }
        }

        private int _adtVersion;

        /// <summary>
        /// Gets and sets the ADT version.
        /// </summary>
        public int AdtVersion
        {
            get { return _adtVersion; }
            set
            {
                _adtVersion = value;
                RaisePropertyChanged("AdtVersion");
            }
        }


        private string _adtDirectory;

        /// <summary>
        /// Gets and sets the directory od ADT.
        /// </summary>
        public string AdtDirectory
        {
            get { return _adtDirectory; }
            set
            {
                _adtDirectory = value;
                RaisePropertyChanged("AdtDirectory");
            }
        }

        private bool _isSystemApp;

        /// <summary>
        /// Gets and sets whether the application is system or not.
        /// </summary>
        public bool IsSystemApp
        {
            get { return _isSystemApp; }
            set
            {
                _isSystemApp = value;
                RaisePropertyChanged("IsSystemApp");
            }
        }




        #endregion

        #region third view

        StopCriteria _selectedStopCriteria;

        /// <summary>
        /// Gets and sets the seelcted stopping criterion.
        /// </summary>
        public StopCriteria SelectedStopCriteria
        {
            get { return _selectedStopCriteria; }
            set
            {
                _selectedStopCriteria = value;
                RaisePropertyChanged("SelectedStopCriteria");
            }
        }

        private string _executor;

        /// <summary>
        /// Gets and sets executor value.
        /// </summary>
        public string Executor
        {
            get { return _executor; }
            set
            {
                _executor = value;
                RaisePropertyChanged("Executor");
            }
        }

        /// <summary>
        /// Gets an array of the values in stop criteria enumeration.
        /// </summary>
        public IEnumerable<StopCriteria> StopCriterias
        {
            get { return Enum.GetValues(typeof(StopCriteria)).Cast<StopCriteria>(); }
        }

        int _stopParameter;

        /// <summary>
        /// Gets and sets the stop parameter.
        /// </summary>
        public int StopParameter
        {
            get { return _stopParameter; }
            set { _stopParameter = value; }
        }

        #endregion

        #region InternalTestCases

        ObservableCollection<TestCase> _internalTestCases;

        /// <summary>
        /// Gets and sets the test case internal content.
        /// </summary>
        [XmlArray]
        public ObservableCollection<TestCase> InternalTestCases
        {
            get
            {
                if (_internalTestCases == null)
                {
                    InternalTestCases = new ObservableCollection<TestCase>();
                }
                return _internalTestCases;
            }
            set
            {
                _internalTestCases = value;
                RaisePropertyChanged("InternalTestCases");
            }
        }

        /// <summary>
        /// Adds an object to end of the collection.
        /// </summary>
        /// <param name="item"></param>
        public void Add(TestCase item)
        {
            InternalTestCases.Add(item);
        }

        /// <summary>
        /// Searches for the specified object.
        /// </summary>
        /// <param name="item"></param>
        /// <returns>
        /// Return an index of the first its occurance in the collection.
        /// </returns>
        public int IndexOf(TestCase item)
        {
            return InternalTestCases.IndexOf(item);
        }

        /// <summary>
        /// Inserts the element in the collection.
        /// </summary>
        /// <param name="index"></param>
        /// <param name="item"></param>
        public void Insert(int index, TestCase item)
        {
            InternalTestCases.Insert(index, item);
        }

        /// <summary>
        /// Removes an element at the specified index in the collection.
        /// </summary>
        /// <param name="index"></param>
        public void RemoveAt(int index)
        {
            InternalTestCases.RemoveAt(index);
        }

        /// <summary>
        /// Gets and sets test case content at specified index in the collection.
        /// </summary>
        /// <param name="index"></param>
        /// <returns>
        /// 
        /// </returns>
        public TestCase this[int index]
        {
            get
            {
                return InternalTestCases[index];
            }
            set
            {
                InternalTestCases[index] = value;
            }
        }

        /// <summary>
        /// Removes all elements from the collection.
        /// </summary>
        public void Clear()
        {
            InternalTestCases.Clear();
        }

        /// <summary>
        /// Determines whether the test case content is in the collection.
        /// </summary>
        /// <param name="item"></param>
        /// <returns>
        /// Returns true if it was found in the collection.
        /// </returns>
        public bool Contains(TestCase item)
        {
            return InternalTestCases.Contains(item);
        }

        /// <summary>
        /// Copies the collection to an array, from the specified position.
        /// </summary>
        /// <param name="array"></param>
        /// <param name="arrayIndex"></param>
        public void CopyTo(TestCase[] array, int arrayIndex)
        {
            InternalTestCases.CopyTo(array, arrayIndex);
        }

        /// <summary>
        /// Gets the number of elements in the collection.
        /// </summary>
        public int Count
        {
            get { return InternalTestCases.Count; }
        }

        /// <summary>
        /// Returns false if it is read-only.
        /// </summary>
        public bool IsReadOnly
        {
            get { return false; }
        }

        /// <summary>
        /// Removes the first element found in the collection.
        /// </summary>
        /// <param name="item"></param>
        /// <returns>
        /// Returns the removed element.
        /// </returns>
        public bool Remove(TestCase item)
        {
            return InternalTestCases.Remove(item);
        }

        /// <summary>
        /// Obtains the enumerator that iterates through the collection.
        /// </summary>
        /// <returns>
        /// Returns the current iterator of the collection.
        /// </returns>
        public IEnumerator<TestCase> GetEnumerator()
        {
            return InternalTestCases.GetEnumerator();
        }     
        #endregion

        Dictionary<string, List<InstrumentedMethod>> _methodsRegistrar;

        /// <summary>
        /// Gets and sets the method registry.
        /// </summary>
        [XmlIgnore]
        public Dictionary<string, List<InstrumentedMethod>> MethodsRegistrar
        {
            get
            {
                if (_methodsRegistrar == null)
                {
                    MethodsRegistrar = new Dictionary<string, List<InstrumentedMethod>>();
                }
                return _methodsRegistrar;
            }
            set
            {
                _methodsRegistrar = value;
                RaisePropertyChanged("MethodsRegistrar");
            }
        }

        #endregion

        #region methods

        /// <summary>
        /// Obtains data for test case.
        /// </summary>
        /// <param name="resultsLoc"></param>
        public void GetData(string resultsLoc)
        {
            InternalTestCases.Clear();
            int testID = 0;

            var Coveredcumulative = new List<InstrumentedMethod>();
            var NotCoveredcumulative = new List<InstrumentedMethod>();

            if (File.Exists(resultsLoc))
            {
                var prevoiousCovered = new List<InstrumentedMethod>();
                var prevoiousNotCovered = new List<InstrumentedMethod>();

                GetTestCases(resultsLoc);

                foreach (var test in InternalTestCases)
                {
                    test.Coverage = 100 * (double)(test.CoveredNumber / (double)(test.CoveredNumber + test.NotCoveredNumber));
                    test.Covered.Clear();
                    test.NotCovered.Clear();
                    test.CumulativeCovered.Clear();
                    test.CumulativeNotCovered.Clear();

                    foreach (var apps in test.Applications)
                    {
                        test.Covered = ExtractUnion<InstrumentedMethod>(test.Covered, apps.Covered);

                        test.NotCovered = ExtractUnion<InstrumentedMethod>(test.NotCovered, apps.NotCovered);

                    }

                    test.CumulativeCovered = ExtractUnion<InstrumentedMethod>(prevoiousCovered, test.Covered);
                    test.CumulativeNotCovered = ExtractUnion<InstrumentedMethod>(prevoiousNotCovered, test.NotCovered);
                    var excepted = test.CumulativeNotCovered.Except(test.CumulativeCovered);
                    test.CumulativeNotCovered = excepted.ToList<InstrumentedMethod>();

                    prevoiousCovered = test.CumulativeCovered;
                    prevoiousNotCovered = test.CumulativeNotCovered;


                    test.ID = testID;
                    testID++;
                }

                var results = GetTestResults(resultsLoc, InternalTestCases.Count());
                foreach (var item in this)
                {
                    {
                        var result = results.Find(n => { return n.TestCase == item.Name; });
                    }
                    try
                    {
                        item.Result = results[0];
                    }
                    catch { }
                }
                var ddd = InternalTestCases;
                InternalTestCases = new ObservableCollection<TestCase>();
                InternalTestCases = ddd;
            }
        }

        private List<T> ExtractUnion<T>(List<T> first, List<T> second)
        {
            return Enumerable.Union<T>(first, second).ToList<T>();
        }


        [DllImport("kernel32.dll", SetLastError = true)]
        private static extern int GetShortPathName(String pathName, StringBuilder shortName, int cbShortName);

        private List<TestResult> GetTestResults(string path, int recordsNo)
        {
            // fix dbf file (add records no)
            byte[] fileBytes = File.ReadAllBytes(Path.GetDirectoryName(path) + @"\" + Path.GetFileNameWithoutExtension(path) + ".dbf");
            if (fileBytes.Count() > 0)
            {
                fileBytes[4] = (byte)((recordsNo      ) & 0xFF);
                fileBytes[5] = (byte)((recordsNo >> 8 ) & 0xFF);
                fileBytes[6] = (byte)((recordsNo >> 16) & 0xFF);
                fileBytes[7] = (byte)((recordsNo >> 24) & 0xFF);
                File.WriteAllBytes(Path.GetDirectoryName(path) + @"\" + Path.GetFileNameWithoutExtension(path) + ".dbf", fileBytes);
            }

            List<TestResult> results = new List<TestResult>();
            string provider = @"vfpoledb";

            StringBuilder sb = new StringBuilder(300);
            int n = GetShortPathName(Path.GetDirectoryName(path), sb, 300);
            string source = sb.ToString();//
            source = @"\";
            string colationSequnce = @"machine";

            Directory.SetCurrentDirectory(sb.ToString());
            using (OleDbConnection con = new OleDbConnection(string.Format(@"Provider={0};Data Source={1}\;Collating Sequence={2};", provider, sb.ToString(), colationSequnce)))
            {
                using (OleDbCommand cm = con.CreateCommand())
                {
                    con.Open();
                    cm.CommandText = "select * from " + Path.GetFileNameWithoutExtension(path);

                    using (OleDbDataReader dr = cm.ExecuteReader())
                    {
                        while (dr.Read())
                        {
                            var result = new TestResult();
                            result.ID = (dr[0] as string).Trim();
                            result.Date = (dr[1] as string).Trim();
                            result.Time = (dr[2] as string).Trim();
                            result.Project = (dr[3] as string).Trim();
                            result.TestCase = (dr[4] as string).Trim();
                            result.Result = (dr[5] as string).Trim();
                            result.Build = (dr[6] as string).Trim();
                            result.Tester = (dr[7] as string).Trim();
                            result.Comment = (dr[8] as string).Trim();
                            result.Description = (dr[9] as string).Trim();
                            result.HW = (dr[10] as string).Trim();
                            result.FilePath = (dr[11] as string).Trim();
                            results.Add(result);
                        }
                    }
                }

                using (StreamWriter sw = new StreamWriter(OutputDirectory + @"\TestResults.txt"))
                {                    
                    sw.WriteLine("#Test cases results");

                    foreach (var item in results)
                    {
                        string itemResult = item.Result;

                        int testResult;
                        switch (itemResult.ToLower())
                        {

                            case "fail":
                                testResult = 0;
                                break;
                            case "pass":
                                testResult = 1;
                                break;
                            default:
                                testResult = 0;
                                break;
                        }
                        sw.WriteLine("{0}\t{1}\t{2}", item.TestCase, item.Date, testResult);                        
                    }
                }
            }            
            return results;
        }

        private List<InstrumentedMethod> GetInstrumentedMethods(string applicationName)
        {
            if (MethodsRegistrar.ContainsKey(applicationName))
            {
                return MethodsRegistrar[applicationName];
            }
            else
            {
                var methods = new List<InstrumentedMethod>();
                var instrumentedMethos = MethodsDirectory + @"\" + applicationName + @"_methods.csv";
                if (File.Exists(instrumentedMethos))
                {
                    MethodsRegistrar.Add(applicationName, methods);
                    using (StreamReader sr = new StreamReader(instrumentedMethos))
                    {
                        sr.ReadLine(); // skip header                    
                        string line;
                        while ((line = sr.ReadLine()) != null)
                        {
                            InstrumentedMethod item = new InstrumentedMethod();
                            var separated = line.Split(';');
                            int id;
                            if (int.TryParse(separated[0].Trim(), out id))
                            {
                                item.ID = id;
                            }
                            item.Method = separated[1];
                            item.Package = separated[2];
                            methods.Add(item);
                            methods.Sort(SortByID);
                        }
                    }
                }
                return methods;
            }
        }

        private static int SortByID(InstrumentedMethod m, InstrumentedMethod n)
        {
            return m.ID - n.ID; ;
        }

        /// <summary>
        /// Method that obtains all the information for a specific test case.
        /// </summary>
        /// <param name="filename"></param>
        public void GetTestCases(string filename)
        {
            InternalTestCases.Clear();
            int testName = 0;
            int applicationName = 1;
            int traceLenhgt = 2;
            int coverage = 3;
            int runtime = 4;
            int covered = 5;
            int notCovered = 6;
            string[] data = File.ReadAllLines(filename);
            string resultsHeader = data[0];            

            if (data != null)
            {
                string testNameValue;
                string applicationNameValue;
                int traceLenhgtValue;
                double coverageValue;
                int runtimeValue;
                string[] coveredList;
                string[] NotCoveredList;

                foreach (string s in data)
                {
                    string[] separated;
                    if (s == data[0]) continue;
                    separated = s.Split(';');

                    testNameValue = separated[testName].Trim();

                    var test = InternalTestCases.FirstOrDefault(n => { return n.Name == testNameValue; });
                    if (test == null)
                    {
                        test = new TestCase();
                        test.Name = testNameValue;
                        InternalTestCases.Add(test);
                    }

                    applicationNameValue = separated[applicationName].Trim();

                    var application = test.Applications.FirstOrDefault(n => { return n.Name == applicationNameValue; });

                    if (application == null)
                    {
                        application = new TestApplication();
                        application.Name = applicationNameValue;
                        test.Applications.Add(application);
                    }

                    if (!int.TryParse(separated[traceLenhgt].Trim(), out traceLenhgtValue))
                    {
                        traceLenhgtValue = -1;
                    }

                    application.TraceLength = traceLenhgtValue;

                    if (!Double.TryParse(separated[coverage].Trim(), out coverageValue))
                    {
                        coverageValue = -1.0;
                    }

                    application.Coverage = coverageValue;

                    if (!int.TryParse(separated[runtime], out runtimeValue))
                    {
                        runtimeValue = -1;
                    }

                    test.Runtime = runtimeValue;

                    coveredList = separated[covered].Trim().Split(' ');

                    if (coveredList.ToString() == "")
                    {
                        break;
                    }

                    List<InstrumentedMethod> methods = GetInstrumentedMethods(applicationNameValue);

                    foreach (string item in coveredList)
                    {
                        item.Trim();

                        if (!string.IsNullOrWhiteSpace(item))
                        {
                            int coveredfunctionID = 0;

                            if (!int.TryParse(item, out coveredfunctionID))
                            {
                                coveredfunctionID = -1;
                            }

                            var method = methods.Find(n => { return n.ID == coveredfunctionID; });

                            if (method == null)
                            {
                                method = new InstrumentedMethod() { ID = coveredfunctionID };
                            }
                            application.Covered.Add(method);
                        }
                    }

                    NotCoveredList = separated[notCovered].Split(' ');

                    foreach (string item in NotCoveredList)
                    {
                        item.Trim();
                        if (!string.IsNullOrWhiteSpace(item))
                        {
                            int coveredfunctionID = 0;

                            if (!int.TryParse(item, out coveredfunctionID))
                            {
                                coveredfunctionID = -1;
                            }

                            var method = methods.Find(n => { return n.ID == coveredfunctionID; });

                            if (method == null)
                            {
                                method = new InstrumentedMethod() { ID = coveredfunctionID };
                            }

                            application.NotCovered.Add(method);
                        }
                    }
                    ID++;
                }
            }
        }
        #endregion
    }
}
