﻿
namespace mockupCirene.ViewModel
{
    using mockupCirene.Helpers;
    using System.Windows.Input;
    using mockupCirene.Model;
    using Microsoft.Win32;

    /// <summary>
    /// This class implements the view model of the configuration process.
    /// </summary>
    public class ConfigurationViewModel : ObservableObject
    {
        public ConfigurationViewModel(TestCases configModel)
        {
            _model = configModel;
        }

        #region Properties
        TestCases _model;

        /// <summary>
        /// Gets and sets the model.
        /// </summary>
        public TestCases Model
        {
            get { return _model; }
            set { _model = value; }
        }

        OpenFileDialog _dialog = new OpenFileDialog();
        #endregion
        #region Commands
        
        #region ChooseAPKFileNameCommand

        /// <summary>
        /// Chooses the APK file name.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseAPKFileName(object parameter)
        {
            var result = OpenDirectoryDialog();
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.MethodsDirectory = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing apk file name is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseAPKFileName(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseAPKFileNameCommand { get { return new RelayCommand<object>(ChooseAPKFileName, CanChooseAPKFileName); } }

        #endregion
        
        #region ChooseTestsDirectory command

        /// <summary>
        /// Chooses the directory of test cases.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseTestsDirectory(object parameter)
        {
            var result = OpenDirectoryDialog();
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.TestDirectory = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing tests directory is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseTestsDirectory(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseTestsDirectoryCommand { get { return new RelayCommand<object>(ChooseTestsDirectory, CanChooseTestsDirectory); } }

        #endregion
        
        #region ChooseProjectDirectory Command

        /// <summary>
        /// Chooses the project directory.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseProjectDirectory(object parameter)
        {
            var result = OpenDirectoryDialog();
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.ProjectDirectory = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing project directory is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseProjectDirectory(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseProjectDirectoryCommand { get { return new RelayCommand<object>(ChooseProjectDirectory, CanChooseProjectDirectory); } }

        #endregion
        
        #region ChooseOutputDirectory Command

        /// <summary>
        /// Chooses the output directory.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseOutputDirectory(object parameter)
        {
            var result = OpenDirectoryDialog();
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.OutputDirectory = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing output directory is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseOutputDirectory(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseOutputDirectoryCommand { get { return new RelayCommand<object>(ChooseOutputDirectory, CanChooseOutputDirectory); } }

        #endregion
        
        #region ChooseChangedFunctionsFileCommand Command

        /// <summary>
        /// Chooses the changed functions file.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseChangedFunctionsFile(object parameter)
        {
            var result = OpenDialog("txt"); ;
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.ChangedFunctionsFile = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing changed functions file is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseChangedFunctionsFile(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseChangedFunctionsFileCommand { get { return new RelayCommand<object>(ChooseChangedFunctionsFile, CanChooseChangedFunctionsFile); } }

        #endregion
        
        #region CoverageConfiguration Command

        /// <summary>
        /// Chooses the coverage configuration.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseCoverageConfiguration(object parameter)
        {

            var result = OpenDirectoryDialog();
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.CoverageConfiguration = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing coverage configuration is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseCoverageConfiguration(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseCoverageConfigurationCommand { get { return new RelayCommand<object>(ChooseCoverageConfiguration, CanChooseCoverageConfiguration); } }

        #endregion
        
        #region SelectionAndPrioritizationConfigFile Command

        /// <summary>
        /// Chooses the configuration file for selection and prioritization.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseSelectionAndPrioritizationConfigFile(object parameter)
        {
            var result = OpenDirectoryDialog();
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.SelectionAndPrioritizationConfigFile = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing configuration file for selection and prioritization is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseSelectionAndPrioritizationConfigFile(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseSelectionAndPrioritizationConfigFileCommand { get { return new RelayCommand<object>(ChooseSelectionAndPrioritizationConfigFile, CanChooseSelectionAndPrioritizationConfigFile); } }

        #endregion

        /// <summary>
        /// Chooses the methods directory.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseMethodsDirectory(object parameter)
        {
            var result = OpenDirectoryDialog();
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.MethodsDirectory = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing methods directory is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseMethodsDirectory(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseMethodsDirectoryCommand { get { return new RelayCommand<object>(ChooseMethodsDirectory, CanChooseMethodsDirectory); } }
       
        #endregion
        #region Dialogs
        private string OpenDirectoryDialog()
        {
            var folderdialog = new System.Windows.Forms.FolderBrowserDialog();
            folderdialog.RootFolder = System.Environment.SpecialFolder.ProgramFiles; //@"c:\Program files\Rt-Executor"
            folderdialog.ShowDialog();
            return folderdialog.SelectedPath.ToString();
        }

        private static string OpenDialog(string extension)
        {
            var filename = "";
            var dialog = new OpenFileDialog();
            dialog.Filter = string.Format("{0} (*.{0})|*.{0}", extension);
            dialog.ValidateNames = true;
            dialog.CheckPathExists = true;
            var showed = dialog.ShowDialog();
            if (showed != null && showed == true)
            {
                filename = dialog.FileName;
            }
            return filename;
        }
        #endregion
    }
}
