﻿
namespace mockupCirene.ViewModel
{
    using System.IO;
    using System.Windows.Input;
    using System.Xml.Serialization;
    using Microsoft.Win32;
    using mockupCirene.Helpers;
    using mockupCirene.Model;

    /// <summary>
    /// This class implements the main view model, with all the steps of the entire testing process.
    /// </summary>
    public class MainViewModel : ObservableObject
    {
        TestCases _model;

        /// <summary>
        /// Gets and sets the model.
        /// </summary>
        public TestCases Model
        {
            get { 
                if(_model == null)
                {
                    Model = new TestCases();
                }
                return _model; }
            set 
            { 
                _model = value;
                RaisePropertyChanged("Model");
            }
        }
        ConfigurationViewModel _configuration;

        /// <summary>
        /// Gets and sets the configuration.
        /// </summary>
        public ConfigurationViewModel Configuration
        {
            get
            {
                if (_configuration == null)
                {
                    Configuration = new ConfigurationViewModel(Model);
                }
                return _configuration;
            }
            set
            {
                _configuration = value;
                RaisePropertyChanged("Configuration");
            }
        }

        CodeInstrumentationViewModel _instrumentation;

        /// <summary>
        /// Gets and sets the instrumentation.
        /// </summary>
        public CodeInstrumentationViewModel Instrumentation
        {
            get 
            {
                if (_instrumentation == null)
                {
                    Instrumentation = new CodeInstrumentationViewModel(Model);
                }
                return _instrumentation;
            }
            set 
            {
                _instrumentation = value;
                RaisePropertyChanged("Instrumentation");
            }
        }

        TestExecutionViewModel _testExec;

        /// <summary>
        /// Gets and sets test execution.
        /// </summary>
        public TestExecutionViewModel TestExecution
        {
            get
            {
                if (_testExec == null)
                {
                    TestExecution = new TestExecutionViewModel(Model);
                }
                return _testExec;
            }
            set
            {
                _testExec = value;
                RaisePropertyChanged("TestExecution");
            }
        }

        ReportViewModel _report;

        /// <summary>
        /// Gets and sets report of execution.
        /// </summary>
        public ReportViewModel Report
        {
            get
            {
                if (_report == null)
                {
                    Report = new ReportViewModel(Model);
                }
                return _report;
            }
            set
            {
                _report = value;
                RaisePropertyChanged("Report");
            }
        }

        /// <summary>
        /// Method which saves the otained state.
        /// </summary>
        /// <param name="parameter"></param>
        public void SaveState(object parameter)
        {            
            var result = SaveDialog("xml");
            if (!string.IsNullOrWhiteSpace(result))
            {
                XmlSerializer mySerializer = new XmlSerializer(typeof(TestCases));
         
                using (StreamWriter myWriter = new StreamWriter(result,false))
                {
                    mySerializer.Serialize(myWriter, Model);
                }
                System.Windows.Forms.MessageBox.Show("Files saved");
            }
        }

        /// <summary>
        /// Method for checking if saving state is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanSaveState(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand SaveStateCommand { get { return new RelayCommand<object>(SaveState, CanSaveState); } }

        /// <summary>
        /// Method which restores the previous state.
        /// </summary>
        /// <param name="parameter"></param>
        public void RestoreState(object parameter)
        {           
            var result = OpenDialog("xml");
         
            if (!string.IsNullOrWhiteSpace(result))
            {
                XmlSerializer mySerializer = new XmlSerializer(typeof(TestCases));            
                using (FileStream myFileStream = new FileStream(result, FileMode.Open))
                {                 
                    Model = (TestCases)mySerializer.Deserialize(myFileStream);
                    Configuration = new ConfigurationViewModel(Model);
                    Instrumentation = new CodeInstrumentationViewModel(Model);
                    TestExecution = new TestExecutionViewModel(Model);
                    Report = new ReportViewModel(Model);
                }
            }
        }

        /// <summary>
        /// Method for checking if restoring the state is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanRestoreState(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand RestoreStateCommand { get { return new RelayCommand<object>(RestoreState, CanRestoreState); } }

        /// <summary>
        /// Method that opens the directory dialog.
        /// </summary>
        /// <returns>
        /// Returns the path of the selected directory.
        /// </returns>
        private string OpenDirectoryDialog()
        {
            var folderdialog = new System.Windows.Forms.FolderBrowserDialog();
            folderdialog.ShowDialog();
            return folderdialog.SelectedPath.ToString();
        }

        private static string OpenDialog(string extension)
        {
            var filename = "";
            var dialog = new OpenFileDialog();
            dialog.Filter = string.Format("{0} (*.{0})|*.{0}", extension);
            dialog.ValidateNames = true;
            dialog.CheckPathExists = true;
            var showed = dialog.ShowDialog();
            if (showed != null && showed == true)
            {
                filename = dialog.FileName;
            }
            return filename;
        }

        private static string SaveDialog(string extension)
        {
            var filename = "";
            var dialog = new SaveFileDialog();
            dialog.Filter = string.Format("{0} (*.{0})|*.{0}", extension);
            dialog.ValidateNames = true;
            dialog.CheckPathExists = true;
            var showed = dialog.ShowDialog();
            if (showed != null && showed == true)
            {
                filename = dialog.FileName;
            }
            return filename;
        }
    }
}