﻿
namespace mockupCirene.ViewModel
{
    using System.Collections.Generic;
    using System.Linq;
    using Microsoft.Win32;
    using mockupCirene.Helpers;
    using mockupCirene.Model;

    /// <summary>
    /// This class implements the report after test execution.
    /// </summary>
    public class ReportViewModel : ObservableObject
    {
        /// <summary>
        /// Construstor for the report view model.
        /// </summary>
        /// <param name="model"></param>
        public ReportViewModel(TestCases model)
        {
            _model = model;
            model.InternalTestCases.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(InternalTestCases_CollectionChanged);
            Model.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(Model_PropertyChanged);
        }

        /// <summary>
        /// Method that raises event based on the specified property.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Model_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "InternalTestCases") 
            {
                RaisePropertyChanged("ModelReversed");
            }
        }

        void InternalTestCases_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            RaisePropertyChanged("ModelReversed");
        }

        TestCases _model;

        /// <summary>
        /// Gets and sets the model.
        /// </summary>
        public TestCases Model
        {
            get { return _model; }
            set 
            { 
                _model = value;
                RaisePropertyChanged("Model");
            }
        }

        /// <summary>
        /// Gets the reversed elements order of the model.
        /// </summary>
        public IEnumerable<TestCase> ModelReversed
        {
            get 
            {
                var reversed = Model.InternalTestCases.Reverse<TestCase>();
                return  reversed; 
            }
        }

        TestCase _selectedTestCase;

        /// <summary>
        /// Gets and sets selected test case.
        /// </summary>
        public TestCase SelectedTestCase
        {
            get { return _selectedTestCase; }
            set
            {
                _selectedTestCase = value;
                RaisePropertyChanged("SelectedTestCase");
            }
        }

        private string OpenDialog(string extension)
        {
            var filename = "";
            var dialog = new OpenFileDialog();
            dialog.Filter = string.Format("{0} (*.{0})|*.{0}", extension);
            dialog.ValidateNames = true;
            dialog.CheckPathExists = true;
            var showed = dialog.ShowDialog();
            if (showed != null && showed == true)
            {
                filename = dialog.FileName;
            }
            return filename;
        }
    }
}