﻿using System;
using System.Windows;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Win32;
using mockupCirene.Helpers;
using mockupCirene.Model;
using mockupCirene.Model.TestModel;
using mockupCirene.Views.ExecutionViews;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Windows.Threading;

namespace mockupCirene.ViewModel
{

    /// <summary>
    /// This class implements the view model of test execution process.
    /// </summary>
    public class TestExecutionViewModel : ObservableObject
    {
        /// <summary>
        /// Constructor of the test execution view model.
        /// </summary>
        /// <param name="model"></param>
        public TestExecutionViewModel(TestCases model)
        {
            Model = model;
        }

        /// <summary>
        /// Declaration of method delegate.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public delegate void ExitedDelegate(object sender, EventArgs e);

        private void StartProccess(string path, string arguments, ExitedDelegate a)
        {
            var proc = new Process();

            proc.StartInfo.FileName = path;

            proc.StartInfo.Arguments = arguments;
            proc.EnableRaisingEvents = true;

            proc.StartInfo.CreateNoWindow = true;
            proc.StartInfo.UseShellExecute = false;
            proc.EnableRaisingEvents = true;

            proc.StartInfo.RedirectStandardError = true;
            proc.Exited += new EventHandler(a);

            proc.Start();

            proc.BeginErrorReadLine();
        }

        void proc_Exited(object sender, EventArgs e)
        {
            if (ActionIndex < ActionsList.Count)
            {
                Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, ActionsList[ActionIndex++]);
            }
            else
            {
                IsTestRunung = false;
            }
        }

        TestCases _model;

        /// <summary>
        /// Gets and sets the model.
        /// </summary>
        public TestCases Model
        {
            get { return _model; }
            set
            {
                _model = value;
                RaisePropertyChanged("Model");
            }
        }

        /// <summary>
        /// Gets and sets the selected stopping criteria.
        /// </summary>
        public StopCriteria SelectedStopCriteria
        {
            get { return Model.SelectedStopCriteria; }
            set
            {
                Model.SelectedStopCriteria = value;

                IsParameter = false;

                RaisePropertyChanged("SelectedStopCriteria");
            }
        }

        bool _isParameter = false;

        /// <summary>
        /// Gets and sets the value of parameter flag.
        /// </summary>
        public bool IsParameter
        {
            get { return _isParameter; }
            set
            {
                _isParameter = value;
                RaisePropertyChanged("IsParameter");
            }
        }

        ObservableCollection<string> _infoField;

        /// <summary>
        /// Gets and sets the information field.
        /// </summary>
        public ObservableCollection<string> InfoField
        {
            get
            {
                if (_infoField == null)
                {
                    InfoField = new ObservableCollection<string>();
                }
                return _infoField;
            }
            set
            {
                _infoField = value;
                RaisePropertyChanged("InfoField");
            }
        }
        #region Commands


        #region Execute Command

        bool _isTestRunung;

        /// <summary>
        /// Gets and sets the value of test running flag.
        /// </summary>
        public bool IsTestRunung
        {
            get { return _isTestRunung; }
            set 
            { 
                _isTestRunung = value;
                RaisePropertyChanged("IsTestRunung");
            }
        }

        /// <summary>
        /// Mmethod that calls method RunExecute
        /// </summary>
        /// <param name="parameter"></param>
        public void Execute(object parameter)
        {
            IsTestRunung = true;

            Browser = new WebBrowser() { };
            var page = new Uri(string.Format(@"http://{0}:8182/", Model.HostAddress.Address), UriKind.RelativeOrAbsolute);
            //var page = new Uri(@"http://www.google.com/");
            Browser.Source = page;//sr.ReadToEnd();      

            RunExecute();

        }

        /// <summary>
        /// Method for checking if execution is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if the test running flag is false.
        /// </returns>
        public bool CanExecute(object parameter)
        {
            return !IsTestRunung;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ExecuteCommand { get { return new RelayCommand<object>(Execute, CanExecute); } }

        /// <summary>
        /// Declaration of method delegate.
        /// </summary>
        public delegate void NextPrimeDelegate();

        private LiveTestingResultWindowView _livePreview;

        /// <summary>
        /// Gets and sets the live preview.
        /// </summary>
        public LiveTestingResultWindowView LivePreview
        {
            get
            {
                if (_livePreview == null)
                {
                    LivePreview = new LiveTestingResultWindowView();
                }
                return _livePreview;
            }
            set
            {
                _livePreview = value;
                RaisePropertyChanged("LivePreview");
            }
        }

        private WebBrowser _browser;

        /// <summary>
        /// Gets and sets the browser.
        /// </summary>
        public WebBrowser Browser
        {
            get { return _browser; }
            set
            {
                _browser = value;
                RaisePropertyChanged("Browser");
            }
        }

        private void RunExecute()
        {

            InfoField.Add("Execution  started");
            EmitConfig(Model);


            EmitRTExecutorConfig(Model.ProjectDirectory);


            SelectionPrioritizationConfigAndRun(Model.SelectionAndPrioritizationConfigFile + @"\SelectionPrioritizationConfigAndRun\SelectionPrioritizationConfigAndRun.tst");
            SelectionPrioritizationRun(Model.SelectionAndPrioritizationConfigFile + @"\SelectionPrioritizationRun\SelectionPrioritizationRun.tst", -1, Model.ResultCsvFile);
            SelectionPrioritizationRunAll(Model.SelectionAndPrioritizationConfigFile + @"\SelectionPrioritizationRunAll\SelectionPrioritizationRunAll.tst");

            CleanAll();



            switch (Model.SelectedStopCriteria)
            {

                case StopCriteria.RunWithoutSelectionAndProiritization:
                    ExecuteRunWithoutSelectionAndPrioritization();

                    break;

                case StopCriteria.RunOnce:
                    ExecuteRunOnce();
                    break;


                case StopCriteria.FindMaximalCoverage:

                    ExecuteFindMaximalCoverage();

                    break;

                default:
                    break;
            }


        }

        public List<Action> ActionsList { get; set; }

        public int ActionIndex { get; set; }

        int cutting;
        double prevCoverage;
        int totalTestCasesNo;

        private void ExecuteFindMaximalCoverage()
        {
            ActionIndex = 0;
            ActionsList = new List<Action>();
            ActionsList.Add(() =>
            {
                InfoField.Add("Starting FindMaximalCoverage");

                // select all
                InfoField.Add("Starting Run_SelectionPrioritizationRunAll");
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                     " \"" +
                                     Path.GetDirectoryName(Model.Executor) +
                                     "\" \"" +
                                     Path.GetFileName(Model.ProjectDirectory) +
                                     @"\Run_SelectionPrioritizationRunAll.txt" + "\"", proc_Exited);
            });

            ActionsList.Add(() =>
            {
                // run all
                InfoField.Add("Starting " + Model.ProjectName);
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                   " \"" +
                                   Path.GetDirectoryName(Model.Executor) +
                                   "\" \"" +
                                   Path.GetFileName(Model.ProjectDirectory) +
                                   @"\" + Model.ProjectName + ".txt" + "\"", proc_Exited);
            });

            ActionsList.Add(() =>
            {

                try
                {
                    RenameOutputs();
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message.ToString());
                }
                Model.GetData(Model.ResultCsvFile);

                SelectionPrioritizationConfigAndRun(Model.SelectionAndPrioritizationConfigFile + @"\SelectionPrioritizationConfigAndRun\SelectionPrioritizationConfigAndRun.tst");

                // do select and config
                InfoField.Add("Starting Run_SelectionPrioritizationConfigAndRun");
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                  " \"" +
                                  Path.GetDirectoryName(Model.Executor) +
                                  "\" \"" +
                                  Path.GetFileName(Model.ProjectDirectory) +
                                  @"\Run_SelectionPrioritizationConfigAndRun.txt" + "\"", proc_Exited);
            });

            ActionsList.Add(() =>
            {
                cutting = 1;
                prevCoverage = 0;
                totalTestCasesNo = Model.InternalTestCases.Count;
                Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, ActionsList[ActionIndex++]);
            });

            ActionsList.Add(() =>
            {
                // select with cutting
                SelectionPrioritizationRun(Model.SelectionAndPrioritizationConfigFile + @"\SelectionPrioritizationRun\SelectionPrioritizationRun.tst", cutting, Model.ResultCsvFile);
                // do selection
                InfoField.Add("Starting Run_SelectionPrioritizationRun");

                StartProccess(Model.Executor, "RT-Executor.exe" +
                                  " \"" +
                                  Path.GetDirectoryName(Model.Executor) +
                                  "\" \"" +
                                  Path.GetFileName(Model.ProjectDirectory) +
                                  @"\Run_SelectionPrioritizationRun.txt" + "\"", proc_Exited);
            });

            ActionsList.Add(() =>
            {
                // run selected
                InfoField.Add("Starting " + Model.ProjectName);
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                   " \"" +
                                   Path.GetDirectoryName(Model.Executor) +
                                   "\" \"" +
                                   Path.GetFileName(Model.ProjectDirectory) +
                                   @"\" + Model.ProjectName + ".txt" + "\"", proc_Exited);
            });

            ActionsList.Add(() =>
            {
                try
                {
                    RenameOutputs();
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message.ToString());
                }

                Model.GetData(Model.ResultCsvFile);

                var testCaseNo = Model.InternalTestCases.Count;

                if (!(testCaseNo >= 1 && (cutting == totalTestCasesNo || prevCoverage >= Model.InternalTestCases[testCaseNo - 1].FullCoverage)))
                {
                    ActionIndex -= 3;
                }

                prevCoverage = Model.InternalTestCases[testCaseNo - 1].FullCoverage;

                cutting++;

                Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, ActionsList[ActionIndex++]);
            });

            ActionsList.Add(() =>
            {
                InfoField.Add("Ended FindMaximalCoverage");

                Model.GetData(Model.ResultCsvFile);

                InfoField.Add("Execution ended");
                IsTestRunung = false;
            });


            Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, ActionsList[ActionIndex++]);
        }

        private void ExecuteRunOnce()
        {
            ActionIndex = 0;
            ActionsList = new List<Action>();
            ActionsList.Add(() =>
            {
                InfoField.Add("Starting RunOnce");
                InfoField.Add("Starting Run_SelectionPrioritizationRunAll");
                // select all
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                        " \"" +
                                        Path.GetDirectoryName(Model.Executor) +
                                        "\" \"" +
                                        Path.GetFileName(Model.ProjectDirectory) +
                                        @"\Run_SelectionPrioritizationRunAll.txt" + "\"", proc_Exited);
            });

            ActionsList.Add(() =>
            {
                InfoField.Add("Starting " + Model.ProjectName);
                // run
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                    " \"" +
                                    Path.GetDirectoryName(Model.Executor) +
                                    "\" \"" +
                                    Path.GetFileName(Model.ProjectDirectory) +
                                    @"\" + Model.ProjectName + ".txt" + "\"", proc_Exited);
            });

            ActionsList.Add(() =>
            {
                try
                {
                    RenameOutputs();
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message.ToString());
                }
                SelectionPrioritizationConfigAndRun(Model.SelectionAndPrioritizationConfigFile + @"\SelectionPrioritizationConfigAndRun\SelectionPrioritizationConfigAndRun.tst");

                // do select and config
                InfoField.Add("Starting Run_SelectionPrioritizationConfigAndRun");
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                    " \"" +
                                    Path.GetDirectoryName(Model.Executor) +
                                    "\" \"" +
                                    Path.GetFileName(Model.ProjectDirectory) +
                                    @"\Run_SelectionPrioritizationConfigAndRun.txt" + "\"", proc_Exited);

            });

            ActionsList.Add(() =>
            {
                // run
                InfoField.Add("Starting " + Model.ProjectName);
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                    " \"" +
                                    Path.GetDirectoryName(Model.Executor) +
                                    "\" \"" +
                                    Path.GetFileName(Model.ProjectDirectory) +
                                    @"\" + Model.ProjectName + ".txt" + "\"", proc_Exited);
            });

            ActionsList.Add(() =>
            {
                try
                {
                    RenameOutputs();
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message.ToString());
                }

                InfoField.Add("Ended RunOnce");


                Model.GetData(Model.ResultCsvFile);

                InfoField.Add("Execution ended");
                IsTestRunung = false;
            });

            Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, ActionsList[ActionIndex++]);
        }

        private void ExecuteRunWithoutSelectionAndPrioritization()
        {
            ActionIndex = 0;
            ActionsList = new List<Action>();
            ActionsList.Add(() =>
            {
                InfoField.Add("Starting RunWithoutSelectionAndProiritization");
                // selection
                InfoField.Add("Starting Run_SelectionPrioritizationRunAll");
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                     " \"" +
                                     Path.GetDirectoryName(Model.Executor) +
                                     "\" \"" +
                                     Path.GetFileName(Model.ProjectDirectory) +
                                     @"\Run_SelectionPrioritizationRunAll.txt" + "\"", proc_Exited);

            });
            ActionsList.Add(() =>
            {
                // run
                InfoField.Add("Starting " + Model.ProjectName);
                StartProccess(Model.Executor, "RT-Executor.exe" +
                                   " \"" +
                                   Path.GetDirectoryName(Model.Executor) +
                                   "\" \"" +
                                   Path.GetFileName(Model.ProjectDirectory) +
                                   @"\" + Model.ProjectName + ".txt" + "\"", proc_Exited);
            });


            ActionsList.Add(() =>
            {
                InfoField.Add("Ended RunWithoutSelectionAndProiritization");
                try
                {
                    RenameOutputs();
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message.ToString());
                }
                Model.GetData(Model.ResultCsvFile);

                InfoField.Add("Execution ended");
                IsTestRunung = false;
            });

            Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, ActionsList[ActionIndex++]);
        }

        private void RenameOutputs()
        {
            string datetime = DateTime.Now.ToShortDateString() + "_" + DateTime.Now.ToLongTimeString();
            datetime = datetime.Replace("/", "-");
            datetime = datetime.Replace(":", "-");
            string newname = "LastProjectResults_" + datetime;
            string newDbfName = newname + ".dbf";
            string newHtmlName = newname + ".html";
            string newResultscsvname = newname + ".csv";

            var htmlfiles = Directory.GetFiles(Model.OutputDirectory, "LastProject *.html");
            File.Move(htmlfiles[0], Model.OutputDirectory + @"\" + newHtmlName);

            var dbffiles = Directory.GetFiles(Model.OutputDirectory, "LastProjectResults.dbf");
            File.Move(dbffiles[0], Model.OutputDirectory + @"\" + newDbfName);
            //var dbffiles = Directory.GetFiles(Model.OutputDirectory, "working.dbf");
            //File.Copy(dbffiles[0], Model.OutputDirectory + @"\" + newDbfName);

            var csvfiles = Directory.GetFiles(Model.OutputDirectory, "result.csv");
            var svcs = Model.OutputDirectory + @"\" + newResultscsvname;
            File.Move(csvfiles[0], svcs);

            Model.ResultCsvFile = svcs;
        }

        private void CleanAll()
        {
            var htmlfiles = Directory.GetFiles(Model.OutputDirectory, "LastProject *.html");
            if (htmlfiles.Count() > 0)
            {
                File.Delete(htmlfiles[0]);
            }

            var dbffiles = Directory.GetFiles(Model.OutputDirectory, "LastProjectResults.dbf");
            if (dbffiles.Count() > 0)
            {
                File.Delete(dbffiles[0]);
            }

            var csvfiles = Directory.GetFiles(Model.OutputDirectory, "result.csv");
            if (csvfiles.Count() > 0)
            {
                File.Delete(csvfiles[0]);
            }
        }

        private void EmitRTExecutorConfig(string filepath)
        {

            // **** make projects for run all ****

            // make bbt project file
            using (StreamWriter sr = new StreamWriter(filepath + @"\Run_SelectionPrioritizationRunAll.bbp", false))
            {
                sr.WriteLine("<RT-RK BlackBox Test Tool Project file>");
                sr.WriteLine("<Project name:Run_SelectionPrioritizationRunAll>");
                sr.WriteLine("<Project database path:" + Model.OutputDirectory + ">");
                sr.WriteLine("<Project HTML path:" + Model.OutputDirectory + ">");
                sr.WriteLine("<Project update:No>");
                sr.WriteLine("<Project repeat:No>");
                sr.WriteLine("[" + @"\SelPriorConfig\SelectionPrioritizationRunAll\SelectionPrioritizationRunAll.tst" + "]");
            }

            // make bbt config for running project from console
            using (StreamWriter sr = new StreamWriter(filepath + @"\Run_SelectionPrioritizationRunAll.txt", false))
            {
                sr.WriteLine("[ProjectInfo]");
                sr.WriteLine(string.Format("Tester = {0}", "CireneUser"));
                sr.WriteLine(string.Format("Build =  {0}", "2013"));
                sr.WriteLine(string.Format("Hardware = {0}", "CireneTestHW"));
                // TODO duplirane putanje do test fajlova (gore su definisane gde se prave testovi, ovde opet gde se pravi bbt config fajl za pokretanje istih) - definisati na jednom mestu
                sr.WriteLine(string.Format("File = {0}", Model.ProjectDirectory + @"\Run_SelectionPrioritizationRunAll.bbp"));
            }


            // **** make projects for run ****

            // make bbt project file
            using (StreamWriter sr = new StreamWriter(Model.ProjectDirectory + @"\Run_SelectionPrioritizationRun.bbp", false))
            {
                sr.WriteLine("<RT-RK BlackBox Test Tool Project file>");
                sr.WriteLine("<Project name:Run_SelectionPrioritizationRun>");
                sr.WriteLine("<Project database path:" + Model.OutputDirectory + ">");
                sr.WriteLine("<Project HTML path:" + Model.OutputDirectory + ">");
                sr.WriteLine("<Project update:No>");
                sr.WriteLine("<Project repeat:No>");
                sr.WriteLine("[" + @"\SelPriorConfig\SelectionPrioritizationRun\SelectionPrioritizationRun.tst" + "]");
            }

            // make bbt config for running project from console
            using (StreamWriter sr = new StreamWriter(filepath + @"\Run_SelectionPrioritizationRun.txt", false))
            {
                sr.WriteLine("[ProjectInfo]");
                sr.WriteLine(string.Format("Tester = {0}", "CireneUser"));
                sr.WriteLine(string.Format("Build =  {0}", "2013"));
                sr.WriteLine(string.Format("Hardware = {0}", "CireneTestHW"));
                // TODO duplirane putanje do test fajlova (gore su definisane gde se prave testovi, ovde opet gde se pravi bbt config fajl za pokretanje istih) - definisati na jednom mestu
                sr.WriteLine(string.Format("File = {0}", Model.ProjectDirectory + @"\Run_SelectionPrioritizationRun.bbp"));
            }


            // **** make projects for config and run ****

            // make bbt project file
            using (StreamWriter sr = new StreamWriter(Model.ProjectDirectory + @"\Run_SelectionPrioritizationConfigAndRun.bbp", false))
            {
                sr.WriteLine("<RT-RK BlackBox Test Tool Project file>");
                sr.WriteLine("<Project name:Run_SelectionPrioritizationConfigAndRun>");
                sr.WriteLine("<Project database path:" + Model.OutputDirectory + ">");
                sr.WriteLine("<Project HTML path:" + Model.OutputDirectory + ">");
                sr.WriteLine("<Project update:No>");
                sr.WriteLine("<Project repeat:No>");
                sr.WriteLine("[" + @"\SelPriorConfig\SelectionPrioritizationConfigAndRun\SelectionPrioritizationConfigAndRun.tst" + "]");
            }

            // make bbt config for running project from console
            using (StreamWriter sr = new StreamWriter(filepath + @"\Run_SelectionPrioritizationConfigAndRun.txt", false))
            {
                sr.WriteLine("[ProjectInfo]");
                sr.WriteLine(string.Format("Tester = {0}", "CireneUser"));
                sr.WriteLine(string.Format("Build =  {0}", "2013"));
                sr.WriteLine(string.Format("Hardware = {0}", "CireneTestHW"));
                // TODO duplirane putanje do test fajlova (gore su definisane gde se prave testovi, ovde opet gde se pravi bbt config fajl za pokretanje istih) - definisati na jednom mestu
                sr.WriteLine(string.Format("File = {0}", Model.ProjectDirectory + @"\Run_SelectionPrioritizationConfigAndRun.bbp"));
            }


            // **** make projects for run selected test plan ****

            // bbt project file already made by selection/prioritization tool

            // make bbt config for running project from console
            using (StreamWriter sr = new StreamWriter(filepath + @"\" + Model.ProjectName + ".txt", false))
            {
                sr.WriteLine("[ProjectInfo]");
                sr.WriteLine(string.Format("Tester = {0}", "CireneUser"));
                sr.WriteLine(string.Format("Build =  {0}", "2013"));
                sr.WriteLine(string.Format("Hardware = {0}", "CireneTestHW"));
                sr.WriteLine(string.Format("File = {0}", Model.ProjectDirectory + @"\" + Model.ProjectName + ".bbp"));
            }
        }

        /// <summary>
        /// Determines whether the resulting dialog of executor is present.
        /// </summary>
        /// <param name="parameter"></param>
        public void ChooseExecutor(object parameter)
        {
            var result = OpenDialog("exe");
            if (!string.IsNullOrWhiteSpace(result))
            {
                Model.Executor = result;
            }
        }

        /// <summary>
        /// Method for checking if choosing the executor is possible.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanChooseExecutor(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ChooseExecutorCommand { get { return new RelayCommand<object>(ChooseExecutor, CanChooseExecutor); } }

        #endregion

        #region Clear Command

        /// <summary>
        /// Removes all elements from collection.
        /// </summary>
        /// <param name="parameter"></param>
        public void Clear(object parameter)
        {
            InfoField.Clear();
        }

        /// <summary>
        /// Method for checking if it is possible to clear elements from collection.
        /// </summary>
        /// <param name="parameter"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanClear(object parameter)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand ClearCommand { get { return new RelayCommand<object>(Clear, CanClear); } }

        #endregion

        /// <summary>
        /// This method checks whether it is possible to load the data and opens the dilaog.
        /// </summary>
        /// <param name="param"></param>
        public void LoadData(object param)
        {
            if (string.IsNullOrWhiteSpace(Model.MethodsDirectory) || File.Exists(Model.MethodsDirectory))
            {
                System.Windows.Forms.MessageBox.Show("Please set Methods directory in Configuratin tab");
            }
            else if (string.IsNullOrWhiteSpace(Model.OutputDirectory) || File.Exists(Model.OutputDirectory))
            {
                System.Windows.Forms.MessageBox.Show("Please set Output directory in Configuratin tab");
            }
            else
            {
                var result = OpenDialog("csv");
                if (!string.IsNullOrWhiteSpace(result))
                {
                    Model.ResultCsvFile = result;
                    Model.GetData(Model.ResultCsvFile);
                }
            }
        }

        /// <summary>
        /// Method for checking if it is possible to load data.
        /// </summary>
        /// <param name="param"></param>
        /// <returns>
        /// Returns true if possible.
        /// </returns>
        public bool CanLoadData(object param)
        {
            return true;
        }

        /// <summary>
        /// Getting the suitable command.
        /// </summary>
        public ICommand LoadDataCommand { get { return new RelayCommand<object>(LoadData, CanLoadData); } }
        
        private string OpenDialog(string extension)
        {
            var filename = "";
            var dialog = new OpenFileDialog();
            dialog.Filter = string.Format("{0} (*.{0})|*.{0}", extension);
            dialog.ValidateNames = true;
            dialog.CheckPathExists = true;
            var showed = dialog.ShowDialog();
            if (showed != null && showed == true)
            {
                filename = dialog.FileName;
            }
            return filename;
        }

        #endregion

        #region Methods

        private void EmitConfig(TestCases model)
        {
            using (StreamWriter coverageConfig = new StreamWriter(Model.CoverageConfiguration + @"\config.txt"))
            {
                coverageConfig.WriteLine("log_level : {0}", model.TestLogLevel);
                coverageConfig.WriteLine("log_location : {0}", model.OutputDirectory + @"\log.txt");
                coverageConfig.WriteLine("res_location : {0}", model.OutputDirectory + @"\result.csv");
                coverageConfig.WriteLine("base_location : {0}", model.OutputDirectory + @"\");
                coverageConfig.WriteLine("methods_location : {0}", model.MethodsDirectory + @"\");
                coverageConfig.WriteLine("host_name : {0}", model.HostAddress);
            }
        }

        private void AssignStep(Step step, string description, string device, string command, int delay, string value)
        {
            step.Description = description;
            step.Device = device;
            step.Command = command;
            step.Value = value;
            step.Delay = delay;
        }

        private void AssignHeader(TSTTest header, string testDirectory, string name)
        {
            header.ID = 999;
            header.Name = name;
            header.Description = "";
            header.Log_file = "\"" + testDirectory + "\"";
        }

        private void SelectionPrioritizationRunAll(string filename)
        {
            InfoField.Add("Emiting SelectionPrioritizationRunAll");

            bool isExists = System.IO.Directory.Exists(Path.GetDirectoryName(filename));
            if (!isExists)
                System.IO.Directory.CreateDirectory(Path.GetDirectoryName(filename));

            using (StreamWriter resultLocation = new StreamWriter(filename))
            {
                var header = new TSTTest();
                var step = new Step();

                AssignHeader(header, "SelectionPrioritizationRunAll.log", "SelectionPrioritizationRunAll");
                resultLocation.Write(header);

                AssignStep(step, "", "SelPrior", "projectName", 0, Model.ProjectName);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "allTCsFile", 0, Model.TestDirectory + @"\" + Model.ProjectName);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "selectionCriteria", 0, ((int)SelectionCriteria.AllTestCases).ToString());
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "cutting", 0, "-1");
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "projectDatabase", 0, Model.OutputDirectory);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "projectHtml", 0, Model.OutputDirectory);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "outputResult", 0, Model.ProjectDirectory);
                resultLocation.Write(step);
            }
        }

        private void SelectionPrioritizationRun(string filename, int cutting, string csvFile)
        {
            InfoField.Add("Emiting SelectionPrioritizationRun");
            bool isExists = System.IO.Directory.Exists(Path.GetDirectoryName(filename));
            if (!isExists)
                System.IO.Directory.CreateDirectory(Path.GetDirectoryName(filename));

            using (StreamWriter resultLocation = new StreamWriter(filename))
            {
                var header = new TSTTest();
                var step = new Step();

                AssignHeader(header, "SelectionPrioritizationRun.log", "SelectionPrioritizationRun");
                resultLocation.Write(header);

                AssignStep(step, "", "SelPrior", "projectName", 0, Model.ProjectName);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "allTCsFile", 0, Model.TestDirectory + @"\" + Model.ProjectName + @"\");
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "changedFnsFile", 0, Model.ChangedFunctionsFile);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "traceLengthsFile", 0, csvFile);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "populateAllFns", 0, Model.MethodsDirectory);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "coverageMatrix", 0, csvFile);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "datesAndResults", 0, Model.OutputDirectory + @"\TestResults.txt");
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "selectionCriteria", 0, ((int)Model.Selection).ToString());
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "priorCriteria", 0, ((int)Model.Prioritization).ToString());
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "cutting", 0, cutting.ToString());
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "projectDatabase", 0, Model.OutputDirectory);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "projectHtml", 0, Model.OutputDirectory);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "outputResult", 0, Model.ProjectDirectory);
                resultLocation.Write(step);

            }
        }

        private void SelectionPrioritizationConfigAndRun(string filename)
        {
            InfoField.Add("Emiting SelectionPrioritizationConfigAndRun");
            bool isExists = System.IO.Directory.Exists(Path.GetDirectoryName(filename));
            if (!isExists)
                System.IO.Directory.CreateDirectory(Path.GetDirectoryName(filename));

            using (StreamWriter resultLocation = new StreamWriter(filename))
            {
                var header = new TSTTest();
                var step = new Step();

                AssignHeader(header, "SelectionPrioritizationConfigAndRun.log", "SelectionPrioritizationConfigAndRun");
                resultLocation.Write(header);

                AssignStep(step, "", "SelPrior", "projectName", 0, Model.ProjectName);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "allTCsFile", 0, Model.TestDirectory + @"\" + Model.ProjectName + @"\");
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "changedFnsFile", 0, Model.ChangedFunctionsFile);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "traceLengthsFile", 0, Model.ResultCsvFile);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "populateAllFns", 0, Model.MethodsDirectory);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "coverageMatrix", 0, Model.ResultCsvFile);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "datesAndResults", 0, Model.OutputDirectory + @"\TestResults.txt");
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "selectionCriteria", 0, ((int)Model.Selection).ToString());
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "priorCriteria", 0, ((int)Model.Prioritization).ToString());
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "projectDatabase", 0, Model.OutputDirectory);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "projectHtml", 0, Model.OutputDirectory);
                resultLocation.Write(step);

                AssignStep(step, "", "SelPrior", "outputResult", 0, Model.ProjectDirectory);
                resultLocation.Write(step);
            }
        }

        #endregion
    }
}
